import { program as app, Command } from '@/program.ts';
import { getConfig, query, writeConfig } from '@/module/index.ts';
import inquirer from 'inquirer';
import util from 'util';

import { saveToken, switchToken, deleteToken, getTokenList } from './login.ts';
const token = new Command('token').description('show token').action(async () => {
  const config = getConfig();
  console.log('token', config.token);
});
const tokenList = new Command('list')
  .description('show token list')
  .option('-r --remove <number>', 'remove token by number')
  .action(async (opts) => {
    const res = await getTokenList();
    if (res.code !== 200) {
      console.error('get token list failed', res.message || '');
      return;
    }
    console.log(util.inspect(res.data.value, { colors: true, depth: 4 }));

    const list = res.data.value || [];
    if (opts.remove) {
      const index = Number(opts.remove) - 1;
      if (index < 0 || index >= list.length) {
        console.log('index out of range');
        return;
      }
      const removeBase = list.splice(index, 1);
      const baseURL = removeBase[0];
      if (baseURL.baseURL) {
        const res = await deleteToken(baseURL?.baseURL);
        if (res.code !== 200) {
          return;
        }
      }
      console.log('delete token success', 'delete', baseURL);
      return;
    }
  });
token.addCommand(tokenList);
app.addCommand(token);

const baseURL = new Command('baseURL')
  .alias('base')
  .alias('registry')
  .description('show baseURL')
  .option('-a, --add <baseURL>', 'add baseURL')
  .option('-r, --remove <number>', 'remove baseURL number')
  .option('-s, --set <number|string>', 'set current baseURL, use number to set from list or string to set')
  .option('-l, --list', 'list baseURL')
  .option('-c, --clear', 'clear baseURL')
  .action(async (opts) => {
    const config = getConfig();
    let list = (config.baseURLList as Array<string>) || [];
    const quineList = (list: string[]) => {
      const newList = new Set(list);
      return Array.from(newList);
    };
    const showList = (list: string[]) => {
      if (list.length === 0) {
        console.log('expand baseURLList is empty');
        return;
      }
      console.log('----current baseURL:' + config.baseURL + '----\n');
      list.forEach((item, index) => {
        console.log(`${index + 1}: ${item}`);
      });
    };
    if (opts.add) {
      list.push(opts.add);
      list = quineList(list);
      console.log('add baseURL success\n');
      writeConfig({ ...config, baseURLList: list });
      showList(list);
      return;
    }
    if (opts.remove) {
      const index = Number(opts.remove) - 1;
      if (index < 0 || index >= list.length) {
        console.log('index out of range');
        return;
      }
      const removeBase = list.splice(index, 1);
      list = quineList(list);
      showList(list);
      writeConfig({ ...config, baseURLList: list });
      removeBase[0] && deleteToken(removeBase[0]);
      return;
    }
    if (opts.set) {
      const isNumber = !isNaN(Number(opts.set));
      let baseURL = '';
      if (isNumber) {
        const index = Number(opts.set) - 1;
        if (index < 0 || index >= list.length) {
          console.log('index out of range');
          return;
        }
        baseURL = list[index];
        writeConfig({ ...config, baseURL: list[index] });
        console.log('set baseURL success:', list[index]);
      } else {
        try {
          new URL(opts.set);
        } catch (error) {
          console.log('invalid baseURL:', opts.set);
          return;
        }
        baseURL = opts.set;
        writeConfig({ ...config, baseURL: opts.set });
        console.log('set baseURL success:', opts.set);
      }
      baseURL && switchToken(baseURL);
      return;
    }
    if (opts.list) {
      showList(list);
      return;
    }
    if (opts.clear) {
      writeConfig({ ...config, baseURLList: [] });
      return;
    }
    console.log('current baseURL:', config.baseURL);
  });
app.addCommand(baseURL);

const setBaseURL = new Command('set').description('set baseURL').action(async () => {
  const config = getConfig();
  const answers = await inquirer.prompt([
    {
      type: 'input',
      name: 'baseURL',
      message: `Enter your baseURL:(current: ${config.baseURL})`,
    },
  ]);
  const baseURL = answers.baseURL;
  writeConfig({ ...config, baseURL });
  baseURL && switchToken(baseURL);
});

baseURL.addCommand(setBaseURL);

// const showQueryURL = new Command('showQueryURL').description('show query URL').action(async () => {
//   console.log("url", query.url);
// });

// app.addCommand(showQueryURL);
